@@ -1,9 +1,6 @@
 #ifndef _PILOT_SOURCE_H_
 #define _PILOT_SOURCE_H_
 
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 #ifdef NeXT
 # include <sys/types.h>
@@ -12,6 +9,8 @@ extern "C" {
 
 #ifdef __EMX__
 # define OS2
+# include <sys/types.h>
+# include <sys/socket.h>
 # include <sys/param.h> /* for htonl .. */
 # define ENOMSG 150
 # define strcasecmp stricmp
@@ -19,7 +18,6 @@ extern "C" {
 
 # include <sys/ioctl.h>
 # include <sys/time.h>
-# include <sys/types.h>
 # include <sys/errno.h>
 # include <time.h>
 # include <fcntl.h>
@@ -46,6 +44,10 @@ extern "C" {
 # include <termios.h>
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include "pi-socket.h"
 #include "pi-macros.h"
 
@@ -67,6 +69,8 @@ struct pi_mac {
   unsigned char *buf;
 };
 
+struct sockaddr;
+
 struct pi_socket {
   struct sockaddr *laddr;
   int laddrlen;
@@ -106,18 +110,18 @@ struct pi_socket {
   int tx_errors;
   int rx_errors;
   char last_tid;
-  int (*socket_connect)(struct pi_socket*, struct sockaddr*, int);
-  int (*socket_listen)(struct pi_socket*, int);
-  int (*socket_accept)(struct pi_socket*, struct sockaddr*, int*);
-  int (*socket_close)(struct pi_socket*);
-  int (*socket_tickle)(struct pi_socket*);
-  int (*socket_bind)(struct pi_socket*, struct sockaddr*, int);
-  int (*socket_send)(struct pi_socket*, void * buf, int len, unsigned int flags);
-  int (*socket_recv)(struct pi_socket*, void * buf, int len, unsigned int flags);
-  int (*serial_close)(struct pi_socket*);
-  int (*serial_changebaud)(struct pi_socket*);
-  int (*serial_write)(struct pi_socket*);
-  int (*serial_read)(struct pi_socket*, int);
+  int (*socket_connect) PI_ARGS((struct pi_socket*, struct sockaddr*, int));
+  int (*socket_listen) PI_ARGS((struct pi_socket*, int));
+  int (*socket_accept) PI_ARGS((struct pi_socket*, struct sockaddr*, int*));
+  int (*socket_close) PI_ARGS((struct pi_socket*));
+  int (*socket_tickle) PI_ARGS((struct pi_socket*));
+  int (*socket_bind) PI_ARGS((struct pi_socket*, struct sockaddr*, int));
+  int (*socket_send) PI_ARGS((struct pi_socket*, void * buf, int len, unsigned int flags));
+  int (*socket_recv) PI_ARGS((struct pi_socket*, void * buf, int len, unsigned int flags));
+  int (*serial_close) PI_ARGS((struct pi_socket*));
+  int (*serial_changebaud) PI_ARGS((struct pi_socket*));
+  int (*serial_write) PI_ARGS((struct pi_socket*));
+  int (*serial_read) PI_ARGS((struct pi_socket*, int));
 #ifdef OS2
   unsigned short os2_read_timeout;
   unsigned short os2_write_timeout;
