@@ -167,5 +167,5 @@ int main(int argc, char *argv[])
   
   dlp_EndOfSync(sd,0);
   pi_close(sd);
-  exit(0);
+  return 0;
 }
